% (April 19, 2016)
% 
% Author:
% Jordan Frecon (jordan.frecon@ens-lyon.fr) 
% --> Contact author Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% 
% Contributors:
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrice Abry (patrice.abry@ens-lyon.fr)
% Laurent Condat (laurent.condat@gipsa-lab.grenoble-inp.fr)
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.
%
%--------------------------------------------------------------------------
% TV On-The-Fly: On-the-fly approximation of the multivariate TV problem  
%                                                                         
% For theoretical aspects please refer to :                               
% J. Frecon, N. Pustelnik, P. Abry, L. Condat                             
% On-The-Fly Approximation of Multivariate Total Variation Minimization   
% IEEE Transactions on Signal Processing,                                 
% Vol. 64, Issue 9, pp. 2355-2364, May. 2016                              
%--------------------------------------------------------------------------
%
% [ X_otf ] = TV_OnTheFly( Y, Lambda, Q)
%
% Computes an on-the-fly approximate solution 'X_otf' to the multivariate
% total variation (TV) problem applied to the multivariate observation 'Y'.
%
% Input:
%   - 'Y' Multivariate observation
%   - 'Lambda' TV regularization parameter
%   - 'Q' Quality parameter for the approximated algorithm
%
% Output:
%   - 'X_otf' Solution of the approximated algorithm
%
% Dependency:
%   - TV_OnTheFly_aux
%       - changePointDetection
%       - changePointDetection_bound
%       - zetaEstimation            
%
function [X,t, Z] = TV_OnTheFly(Y, Lambda, Q)


Init = 0;
tic;
N       = length(Y);
X       = [];
Z       = [];
k0      = 1;


% - Approximated Solution
[temp, aux, k0,kOut, Ztmp]  = TV_OnTheFly_aux(Y(:,k0), Lambda, Q, k0, k0, Init);
X                           = horzcat(X,temp);
Z                           = horzcat(Z,Ztmp);
kp                          = kOut;
aux.sigma2                  = ones(Q,1)*std(Y,0,2)';


for(k=2:N)
        
    while(kp <= k);
 
        if(kp < N)
            [temp, aux, k0,kOut, Ztmp]    = TV_OnTheFly_aux(Y(:,kp), Lambda, Q, kp, k0, Init, aux);
        else
            [temp, aux, k0,kOut, Ztmp]    = TV_OnTheFly_aux(Y(:,kp), Lambda, Q, kp, k0, Init, aux, N);
        end
        X   = horzcat(X, temp);
        Z   = horzcat(Z, Ztmp);
        kp  = kOut;
   
    end
    
end


    
X(:,end+1)      = X(:,end);
    
t = toc;