% (April 19, 2016)
% 
% Author:
% Jordan Frecon (jordan.frecon@ens-lyon.fr) 
% --> Contact author Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% 
% Contributors:
% Nelly Pustelnik (nelly.pustelnik@ens-lyon.fr)
% Patrice Abry (patrice.abry@ens-lyon.fr)
% Laurent Condat (laurent.condat@gipsa-lab.grenoble-inp.fr)
% 
% This software is governed by the CeCILL license under French law and
% abiding by the rules of distribution of free software.  You can  use,
% modify and/ or redistribute the software under the terms of the CeCILL
% license as circulated by CEA, CNRS and INRIA at the following URL
% "http://www.cecill.info".
% 
% As a counterpart to the access to the source code and  rights to copy,
% modify and redistribute granted by the license, users are provided only
% with a limited warranty  and the software's author,  the holder of the
% economic rights,  and the successive licensors  have only  limited
% liability.
% 
% In this respect, the user's attention is drawn to the risks associated
% with loading,  using,  modifying and/or developing or reproducing the
% software by the user in light of its specific status of free software,
% that may mean  that it is complicated to manipulate,  and  that  also
% therefore means  that it is reserved for developers  and  experienced
% professionals having in-depth computer knowledge. Users are therefore
% encouraged to load and test the software's suitability as regards their
% requirements in conditions enabling the security of their systems and/or
% data to be ensured and,  more generally, to use and operate it in the
% same conditions as regards security.
% 
% The fact that you are presently reading this means that you have had
% knowledge of the CeCILL license and that you accept its terms.
%
%--------------------------------------------------------------------------
% TV On-The-Fly: On-the-fly approximation of the multivariate TV problem  
%                                                                         
% For theoretical aspects please refer to :                               
% J. Frecon, N. Pustelnik, P. Abry, L. Condat                             
% On-The-Fly Approximation of Multivariate Total Variation Minimization   
% IEEE Transactions on Signal Processing,                                 
% Vol. 64, Issue 9, pp. 2355-2364, May. 2016                              
%--------------------------------------------------------------------------
%
% demo_TV_OnThe_Fly.m
%
% Provides one denoising example of a multivariate signal (with M components)
% 
% Two setting are displayed:
% 1) Offline setting, showing the solution for a given observation.
% 2) Online setting, showing the solution as the observation stream incomes.
% Both solutions are equal. The online setting allows the user to visualize
% the on-the-fly behavior of the solution.
% 
% The main functions are TV_OnTheFly.m and TV_OnTheFly_stream.m.
%


mydir  = which('demo_TV_OnTheFly.m');
idcs   = strfind(mydir,'/');
newdir = mydir(1:idcs(end));       
addpath(genpath(newdir));
grey = .6.*[1 1 1];


% Signal
M = 3;
X = [ones(M,250), 3*ones(M,200), 2*ones(M,300), 0*ones(M,250)];
N = length(X);
s = randn(M,1)*ones(1,N);
Y = X + s.*randn(M,N);


% TV Parameters
lambda  = 20;       %Regularization parameter
Q       = 20;       %The higher Q, the higher the quality and execution time




%% OFFLINE RESULT
% Shows the corresponding solution in an offline setting


% TV Algorithm
X_otf   = TV_OnTheFly(Y,lambda,Q);


% Display
figure(1);clf
for mm=1:M
    subplot(M,1,mm)
    plot(Y(mm,:),'color',grey); hold on;
    plot(X(mm,:),'k','linewidth',3);
    plot(X_otf(mm,:),'-.r','linewidth',3);
    axis([0 N min(Y(mm,:)) max(Y(mm,:))]);
    grid on;
    
    if mm == 1
       legend('Observation','True Signal','On-the-fly Solution');
       title('Offline result');
    end
end
drawnow;


%% ONLINE RESULT
% Shows how the solution is computed in an online setting, i.e. for an
% incoming stream of data 'Y'.


param.X     = X;    %[optional] true signal
param.fps   = 2^8;  %[optional] #frames per second


X_otf = TV_OnTheFly_stream( Y, lambda, Q, param);

